OUTPUT_REFERENCE_COLUMN = "output"
OUTPUT_MESSAGE_REFERENCE_COLUMN = "message"
NULL_SUFFIX = "_void"
FILTER_SUFFIX = "_filter"
INPUT_OUTPUT_VALUE_CONNECTOR_NAME = "output_value"
FLOW_NODE_NAME = "flowv0"
